"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const DEFAULT_CONFIG_FILE_NAME = "regconfig.json";
const ESCAPE_REGEXP = /\$(\$[^\$]+)/g;
const PLACEHOLDER_REGEXP = /\$([^\$]+)/g;
const PLACEHOLDER_REGEXP_BRACE = /\$\{([^\$\{\}]+)\}/g;
function expandPlaceholders(x) {
    if (typeof x === "object") {
        Object.keys(x).forEach(k => {
            x[k] = expandPlaceholders(x[k]);
        });
        return x;
    }
    else if (Array.isArray(x)) {
        return x.map(item => expandPlaceholders(x));
    }
    else if (typeof x === "string") {
        if (ESCAPE_REGEXP.test(x)) {
            return x.replace(ESCAPE_REGEXP, (_, g1) => g1);
        }
        else if (PLACEHOLDER_REGEXP_BRACE.test(x)) {
            return x.replace(PLACEHOLDER_REGEXP_BRACE, (_, g1) => process.env[g1]);
        }
        else if (PLACEHOLDER_REGEXP.test(x)) {
            return x.replace(PLACEHOLDER_REGEXP, (_, g1) => process.env[g1]);
        }
        else {
            return x;
        }
    }
    else {
        return x;
    }
}
class ConfigManager {
    get defaultConfigFileName() {
        return DEFAULT_CONFIG_FILE_NAME;
    }
    replaceEnvValue(rawConfig) {
        if (!rawConfig.plugins)
            return rawConfig;
        const plugins = Object.assign({}, rawConfig.plugins);
        if (!!plugins["__replaced__"])
            return rawConfig;
        expandPlaceholders(plugins);
        plugins["__replaced__"] = true;
        return Object.assign({}, rawConfig, { plugins });
    }
    readConfig(configFileName = DEFAULT_CONFIG_FILE_NAME) {
        const defaultCoreConfig = {
            workingDir: path.join(process.cwd(), "./reg"),
            actualDir: "actual",
            expectedDir: "expected",
        };
        let readResult, readJsonObj;
        try {
            readResult = fs.readFileSync(this._getConfigPath(configFileName), "utf8");
            readJsonObj = JSON.parse(readResult);
        }
        catch (e) {
            // TODO logging
        }
        if (readJsonObj) {
            return {
                readUserConfig: true,
                config: {
                    core: Object.assign({}, defaultCoreConfig, readJsonObj.core),
                    plugins: readJsonObj.plugins,
                },
            };
        }
        else {
            return {
                readUserConfig: false,
                config: {
                    core: defaultCoreConfig,
                },
            };
        }
    }
    writeConfig(config, configFileName = DEFAULT_CONFIG_FILE_NAME) {
        fs.writeFileSync(this._getConfigPath(configFileName), JSON.stringify(config, null, 2), "utf8");
    }
    _getConfigPath(configFileName) {
        return path.resolve(process.cwd(), configFileName);
    }
}
exports.ConfigManager = ConfigManager;
const configManager = new ConfigManager();
exports.default = configManager;
