import { RegLogger, LogLevel } from "reg-suit-util";
import { ConfigManager } from "./config-manager";
import { KeyGeneratorPlugin, NotifierPlugin, Plugin, PublisherPlugin, CoreConfig, CreateQuestionsOptions, RegSuitConfiguration, ComparisonResult } from "reg-suit-interface";
export interface PluginMetadata {
    moduleId: string;
    [key: string]: any;
}
export interface StepResultAfterExpectedKey {
    expectedKey: string | null;
}
export interface StepResultAfterComparison extends StepResultAfterExpectedKey {
    comparisonResult: ComparisonResult;
}
export interface StepResultAfterActualKey extends StepResultAfterComparison {
    actualKey: string;
}
export interface StepResultAfterPublish extends StepResultAfterActualKey {
    reportUrl: string | null;
}
export declare class RegSuitCore {
    _keyGenerator?: KeyGeneratorPlugin<any>;
    _publisher?: PublisherPlugin<any>;
    _notifiers: NotifierPlugin<any>[];
    noEmit: boolean;
    logger: RegLogger;
    _config: RegSuitConfiguration;
    _configManager: ConfigManager;
    _pluginHolders: PluginMetadata[];
    constructor(opt?: {
        logLevel?: LogLevel;
        noEmit?: boolean;
    });
    _loadPlugins(): void;
    _loadPlugin(name: string): void;
    createQuestions(opt: CreateQuestionsOptions): {
        name: string;
        questions: any[];
        prepare: (inquireResult: any) => Promise<any>;
        configured: any;
    }[];
    persistMergedConfig(opt: {
        core?: CoreConfig;
        pluginConfigs: {
            name: string;
            config: any;
        }[];
    }, confirm: (newConfig: RegSuitConfiguration) => Promise<boolean>): Promise<void>;
    init(configFileName?: string): void;
    _initPlugin<S, P extends Plugin<S>>(targetPlugin: P, metadata: PluginMetadata): P;
    _initKeyGenerator(): void;
    _initPublisher(): void;
    _initNotifiers(): void;
    _loadConfig(configFileName?: string): RegSuitConfiguration;
    getDirectoryInfo(configFileName?: string): {
        actualDir: string;
        expectedDir: string;
    };
    runAll(): Promise<StepResultAfterPublish>;
    getExpectedKey(): Promise<StepResultAfterExpectedKey>;
    compare(ctx: StepResultAfterExpectedKey): Promise<StepResultAfterComparison>;
    getActualKey(ctx: StepResultAfterComparison): Promise<StepResultAfterActualKey>;
    fetch(ctx: StepResultAfterExpectedKey): Promise<StepResultAfterExpectedKey>;
    publish(ctx: StepResultAfterActualKey): Promise<StepResultAfterPublish>;
    notify(ctx: StepResultAfterPublish): Promise<StepResultAfterPublish>;
}
